# Присутствие. Сервисное сообщение для извещения сервера о присутствии клиента online;
PRESENCE = 'presence'

# Проверка присутствия. Сервисное сообщение от сервера для проверки присутствии клиента online;
PROBE = 'prоbe'

# Простое сообщение пользователю или в чат;
MSQ = 'msg'

# Отключение от сервера;
QUIT = 'quit'

# Авторизация на сервере;
AUTHENTICATE = 'authenticate'

# Присоединиться к чату;
JOIN = 'join'

# Покинуть чат.
LEAVE = 'leave'
